<?php
namespace App\Utils;

use DOMDocument;

class XMLTransformer
{
    private $resultXML;
    private $resultData = [];
    private $usedData = [];
    private $defaultElements = ["Products", "Product", "ID", "Title", "EAN", "Code", "Price", "DPH", "Description",
        "Weight", "Producer", "Images", "Image", "URL", "Parameters", "Parameter", "Value", "Variations", "Variation"];
    private $rootElement;
    private $secondRootElement;

    public function transformToResultData($userData, $XMLTemplate) {
        $usedUserElements = $this->getUsedUserElements($XMLTemplate, $XMLTemplate);
        $this->usedData = $usedUserElements;
        $this->getXMLData($userData, 0);
        if (empty($this->getResultData())) {
            return null;
        }
        $xml = $this->XMLTransformation($this->getResultData());
        $this->setResultXML($xml);
    }

    public function getUsedUserElements($xml, $xml2)
    {
        $defaultElements = $this->defaultElements;
        foreach ($xml as $index => $element) {
            if (in_array($element, $defaultElements)) {
                unset($xml[$index]);
                continue;
            }
            $xml[$index] = $this->searchParent($xml2, $index, $element);
            if ($xml[$index] == $element.'Product') {
                $this->secondRootElement = substr_replace($element,"" ,-1);
            }
            if ($xml[$index] == $element.'Products') {
                $this->rootElement = substr_replace($element,"" ,-1);
            }
            if ($this->inVariation($xml2, $index)) {
                $xml[$index] = $xml[$index].'Var';
            }
        }

        return $xml;
    }

    private function searchParent($xml, $index, $name)
    {
        for($i = $index; $i >= 0; $i--) {
            if (in_array($xml[$i], $this->defaultElements)) {
                return $name.$xml[$i];
            }
        }
        return null;
    }

    private function inVariation($xml, $index) {
        for($i = $index; $i >= 0; $i--) {
            if ($xml[$i] === 'Variation') {
                return true;
            }
        }
        return false;
    }

    public function getXMLData($xml, $index, $path = null) {
        while ($xml->read()) {
            if ($xml->nodeType == \XMLReader::END_ELEMENT) {
                return;
            } else if ($xml->nodeType == \XMLReader::ELEMENT) {
                $newPath = $path.$xml->name;
                if ($this->secondRootElement == $newPath) {
                    $index++;
                }

                if (!$xml->isEmptyElement) {
                    $this->getXMLData($xml, $index, $newPath.'/');
                }

            }

            else if($xml->nodeType == \XMLReader::TEXT)
            {
                $this->addXMLValues($xml->value, $index, $path);
            }
        }
    }

    private function addXMLValues($text, $index, $path) {
        $foundElements = $this->inUsedDataValue($path);
        if (sizeof($foundElements) < 1) {
            return;
        }
        $result = $this->getResultData();
        if (!isset($result[$index][0])) {
            $result[$index][0] = [];
        }
        foreach ($foundElements as $element) {
            if (substr($element, -3) == 'Var') {
                $result[$index][0][] = ['text' => $text, 'newElement' => $element];
            } else {
                $result[$index][] = ['text' => $text, 'newElement' => $element];
            }
        }
        $this->setResultData($result);
    }

    private function inUsedDataValue($value) {
        $foundElements = [];
        foreach ($this->usedData as $element) {
            $tmp = explode('_', $element);
            $newEl = $tmp[sizeof($tmp)-1];
            unset($tmp[sizeof($tmp)-1]);
            if ($value == implode('_', $tmp) || $value == implode('_', $tmp).'/') {
                $foundElements[] = $newEl;
            }
        }
        return $foundElements;
    }

    public function XMLTransformation($xml) {
        $doc  = new DomDocument();
        $doc->formatOutput   = true;

        $root = $doc->createElement('Products');
        $root = $doc->appendChild($root);

        foreach ($xml as $elements) {
            $container = $doc->createElement('Product');
            foreach ($elements as $index => $element) {
                if (empty($element)) {
                    continue;
                }
                $child = $doc->createElement($element['newElement']);
                $child = $container->appendChild($child);
                $value = $doc->createTextNode($element['text']);
                $value = $child->appendChild($value);
            }
            $root->appendChild($container);

        }
        return $doc->saveXML();
    }

    /**
     * @return array
     */
    public function getResultData(): array
    {
        return $this->resultData;
    }

    /**
     * @param array $resultData
     */
    public function setResultData(array $resultData): void
    {
        $this->resultData = $resultData;
    }

    /**
     * @return mixed
     */
    public function getResultXML()
    {
        return $this->resultXML;
    }

    /**
     * @param mixed $resultXML
     */
    public function setResultXML($resultXML): void
    {
        $this->resultXML = $resultXML;
    }


}

